


import pandas as pd
from numpy_typing import np, ax, ax
import matplotlib.pyplot as plt

from .B_Model_AbstractModel import Model

from . import _Utils_Color as C
from ._Utils_Color import prntC



class Trainer:
    """"
    Template Trainer class (use Inherit).
    Manage the whole training of a model.

    Attributes :
    ------------

    CTX : dict
        The hyperparameters context

    model : type[Model]

    Methods :
    ---------

    run():
        Run the whole training pipeline
        and return metrics about the model's performance

    train(): Abstract
        Manage the training loop

    eval(): Abstract
        Evaluate the model and return metrics

    """


    ###################################################
    # Initialization
    ###################################################

    def __init__(self, CTX:dict, model:"type[Model]"):

        pass

    def viz_model(self, filename:str):
        try:
            self.model.visualize(filename)
        except Exception as e:

            prntC(C.WARNING, "Visualization of the model failed")
            prntC(C.RED, e)
            prntC()

    ###################################################
    # Traier main loop
    ###################################################

    def run(self):
        """
        Run the whole training pipeline
        and return metrics about the model's performance

        Returns:
        --------

        metrics : dict
            The metrics dictionary representing model's performance
        """
        if (self.CTX["EPOCHS"] > 0):
            self.train()
        else:
            self.load()

        return self.eval()


    ###################################################
    # Save and load model
    ###################################################

    def save(self):
        """
        Save the model's weights in the _Artifacts folder
        """
        raise NotImplementedError

    def load(self):
        """
        Load the model's weights from the _Artifacts folder
        """
        raise NotImplementedError



    def train(self):
        """
        Manage the training loop.
        Testing is also done here.
        At the end, you can save your best model.
        """
        raise NotImplementedError

    ###################################################
    # Evaluation
    ###################################################

    def eval(self):
        """
        Evaluate the model and return metrics

        Returns:
        --------

        metrics : dict
            The metrics dictionary of the model's performance
        """
        raise NotImplementedError

        # example of return metrics:
        return {
            "Accuracy": 0.5,
            "False-Positive": 0.5,
            "False-Negative": 0.5
        }


















